@@ -1,15 +1,16 @@
 # Copyright (c) 2006-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2013-2014 Google, Inc.
 # Copyright (c) 2013 buck@yelp.com <buck@yelp.com>
-# Copyright (c) 2014-2019 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Brett Cannon <brett@python.org>
 # Copyright (c) 2014 Arun Persaud <arun@nubati.net>
 # Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
 # Copyright (c) 2016 Moises Lopez <moylop260@vauxoo.com>
 # Copyright (c) 2017-2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
-# Copyright (c) 2018-2019 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2018-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
 # Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
 # Copyright (c) 2019 Bruno P. Kinoshita <kinow@users.noreply.github.com>
+# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -51,15 +52,12 @@ class BaseChecker(OptionsProviderMixIn):
 
     def __gt__(self, other):
         """Permit to sort a list of Checker by name."""
-        return "{}{}".format(self.name, self.msgs).__gt__(
-            "{}{}".format(other.name, other.msgs)
-        )
+        return f"{self.name}{self.msgs}".__gt__(f"{other.name}{other.msgs}")
 
     def __repr__(self):
         status = "Checker" if self.enabled else "Disabled checker"
-        return "{} '{}' (responsible for '{}')".format(
-            status, self.name, "', '".join(self.msgs.keys())
-        )
+        msgs = "', '".join(self.msgs.keys())
+        return f"{status} '{self.name}' (responsible for '{msgs}')"
 
     def __str__(self):
         """This might be incomplete because multiple class inheriting BaseChecker
@@ -80,15 +78,15 @@ class BaseChecker(OptionsProviderMixIn):
         result += "Verbatim name of the checker is ``%s``.\n\n" % self.name
         if doc:
             # Provide anchor to link against
-            result += get_rst_title("{} Documentation".format(checker_title), "^")
+            result += get_rst_title(f"{checker_title} Documentation", "^")
             result += "%s\n\n" % cleandoc(doc)
         # options might be an empty generator and not be False when casted to boolean
         options = list(options)
         if options:
-            result += get_rst_title("{} Options".format(checker_title), "^")
+            result += get_rst_title(f"{checker_title} Options", "^")
             result += "%s\n" % get_rst_section(None, options)
         if msgs:
-            result += get_rst_title("{} Messages".format(checker_title), "^")
+            result += get_rst_title(f"{checker_title} Messages", "^")
             for msgid, msg in sorted(
                 msgs.items(), key=lambda kv: (_MSG_ORDER.index(kv[0][0]), kv[1])
             ):
@@ -96,7 +94,7 @@ class BaseChecker(OptionsProviderMixIn):
                 result += "%s\n" % msg.format_help(checkerref=False)
             result += "\n"
         if reports:
-            result += get_rst_title("{} Reports".format(checker_title), "^")
+            result += get_rst_title(f"{checker_title} Reports", "^")
             for report in reports:
                 result += ":%s: %s\n" % report[:2]
             result += "\n"
@@ -118,18 +116,14 @@ class BaseChecker(OptionsProviderMixIn):
         checker.
 
         :raises InvalidMessageError: If the checker id in the messages are not
-        always the same. """
+        always the same."""
         checker_id = None
         existing_ids = []
         for message in self.messages:
             if checker_id is not None and checker_id != message.msgid[1:3]:
                 error_msg = "Inconsistent checker part in message id "
-                error_msg += "'{}' (expected 'x{checker_id}xx' ".format(
-                    message.msgid, checker_id=checker_id
-                )
-                error_msg += "because we already had {existing_ids}).".format(
-                    existing_ids=existing_ids
-                )
+                error_msg += f"'{message.msgid}' (expected 'x{checker_id}xx' "
+                error_msg += f"because we already had {existing_ids})."
                 raise InvalidMessageError(error_msg)
             checker_id = message.msgid[1:3]
             existing_ids.append(message.msgid)
@@ -171,8 +165,8 @@ class BaseChecker(OptionsProviderMixIn):
         for message_definition in self.messages:
             if message_definition.msgid == msgid:
                 return message_definition
-        error_msg = "MessageDefinition for '{}' does not exists. ".format(msgid)
-        error_msg += "Choose from {}.".format([m.msgid for m in self.messages])
+        error_msg = f"MessageDefinition for '{msgid}' does not exists. "
+        error_msg += f"Choose from {[m.msgid for m in self.messages]}."
         raise InvalidMessageError(error_msg)
 
     def open(self):
